//          Copyright Naoki Shibata 2010 - 2019.
// Distributed under the Boost Software License, Version 1.0.
//    (See accompanying file LICENSE.txt or copy at
//          http://www.boost.org/LICENSE_1_0.txt)

#ifndef __SLEEFQUAD_H__
#define __SLEEFQUAD_H__

#include "sleef.h"

#ifdef __cplusplus
extern "C"
{
#endif

#if (defined(__MINGW32__) || defined(__MINGW64__) || defined(__CYGWIN__) || defined(_MSC_VER)) && !defined(SLEEF_STATIC_LIBS)
#ifdef IMPORT_IS_EXPORT
#define IMPORT __declspec(dllexport)
#else // #ifdef IMPORT_IS_EXPORT
#define IMPORT __declspec(dllimport)
#if (defined(_MSC_VER))
#pragma comment(lib,"sleefquad.lib")
#endif // #if (defined(_MSC_VER))
#endif // #ifdef IMPORT_IS_EXPORT
#else // #if (defined(__MINGW32__) || defined(__MINGW64__) || defined(__CYGWIN__) || defined(_MSC_VER)) && !defined(SLEEF_STATIC_LIBS)
#define IMPORT
#endif // #if (defined(__MINGW32__) || defined(__MINGW64__) || defined(__CYGWIN__) || defined(_MSC_VER)) && !defined(SLEEF_STATIC_LIBS)

//

#if !defined(Sleef_quad_DEFINED)
#define Sleef_quad_DEFINED
#if defined(ENABLEFLOAT128)
typedef __float128 Sleef_quad;
#else
typedef struct { uint64_t x, y; } Sleef_quad;
#endif
#endif

#if !defined(Sleef_quad1_DEFINED)
#define Sleef_quad1_DEFINED
typedef union {
  struct {
    Sleef_quad x;
  };
  Sleef_quad s[1];
} Sleef_quad1;
#endif

#if !defined(Sleef_quad2_DEFINED)
#define Sleef_quad2_DEFINED
typedef union {
  struct {
    Sleef_quad x, y;
  };
  Sleef_quad s[2];
} Sleef_quad2;
#endif

#if !defined(Sleef_quad4_DEFINED)
#define Sleef_quad4_DEFINED
typedef union {
  struct {
    Sleef_quad x, y, z, w;
  };
  Sleef_quad s[4];
} Sleef_quad4;
#endif

#if !defined(Sleef_quad8_DEFINED)
#define Sleef_quad8_DEFINED
typedef union {
  Sleef_quad s[8];
} Sleef_quad8;
#endif

#if defined(__ARM_FEATURE_SVE) && !defined(Sleef_quadx_DEFINED)
#define Sleef_quadx_DEFINED
typedef union {
  Sleef_quad s[32];
} Sleef_quadx;
#endif

//

IMPORT Sleef_quad1 Sleef_strtoq(const char *str, char **endptr, int base);
IMPORT void Sleef_qtostr(char *s, int n, Sleef_quad1 a, int base);

//
#ifdef __STDC__
IMPORT CONST Sleef_quad1 Sleef_addq1_u05purec(Sleef_quad1, Sleef_quad1);
IMPORT CONST Sleef_quad1 Sleef_subq1_u05purec(Sleef_quad1, Sleef_quad1);
IMPORT CONST Sleef_quad1 Sleef_mulq1_u05purec(Sleef_quad1, Sleef_quad1);
IMPORT CONST Sleef_quad1 Sleef_divq1_u05purec(Sleef_quad1, Sleef_quad1);
IMPORT CONST Sleef_quad1 Sleef_negq1_purec(Sleef_quad1);
IMPORT CONST Sleef_quad1 Sleef_sqrtq1_u05purec(Sleef_quad1);
IMPORT CONST int32_t Sleef_cmpltq1_purec(Sleef_quad1, Sleef_quad1);
IMPORT CONST int32_t Sleef_cmpleq1_purec(Sleef_quad1, Sleef_quad1);
IMPORT CONST int32_t Sleef_cmpgtq1_purec(Sleef_quad1, Sleef_quad1);
IMPORT CONST int32_t Sleef_cmpgeq1_purec(Sleef_quad1, Sleef_quad1);
IMPORT CONST int32_t Sleef_cmpeqq1_purec(Sleef_quad1, Sleef_quad1);
IMPORT CONST int32_t Sleef_cmpneqq1_purec(Sleef_quad1, Sleef_quad1);
IMPORT CONST int32_t Sleef_unordq1_purec(Sleef_quad1, Sleef_quad1);
IMPORT CONST double Sleef_cast_to_doubleq1_purec(Sleef_quad1);
IMPORT CONST Sleef_quad1 Sleef_cast_from_doubleq1_purec(double);
IMPORT CONST Sleef_quad1 Sleef_sinq1_u10purec(Sleef_quad1);
IMPORT CONST Sleef_quad1 Sleef_cosq1_u10purec(Sleef_quad1);
IMPORT CONST Sleef_quad1 Sleef_tanq1_u10purec(Sleef_quad1);
IMPORT CONST Sleef_quad1 Sleef_asinq1_u10purec(Sleef_quad1);
IMPORT CONST Sleef_quad1 Sleef_acosq1_u10purec(Sleef_quad1);
IMPORT CONST Sleef_quad1 Sleef_atanq1_u10purec(Sleef_quad1);
IMPORT CONST Sleef_quad1 Sleef_expq1_u10purec(Sleef_quad1);
IMPORT CONST Sleef_quad1 Sleef_exp2q1_u10purec(Sleef_quad1);
IMPORT CONST Sleef_quad1 Sleef_exp10q1_u10purec(Sleef_quad1);
IMPORT CONST Sleef_quad1 Sleef_expm1q1_u10purec(Sleef_quad1);
IMPORT CONST Sleef_quad1 Sleef_logq1_u10purec(Sleef_quad1);
IMPORT CONST Sleef_quad1 Sleef_log2q1_u10purec(Sleef_quad1);
IMPORT CONST Sleef_quad1 Sleef_log10q1_u10purec(Sleef_quad1);
IMPORT CONST Sleef_quad1 Sleef_log1pq1_u10purec(Sleef_quad1);
#endif
#ifdef FP_FAST_FMA
IMPORT CONST Sleef_quad1 Sleef_addq1_u05purecfma(Sleef_quad1, Sleef_quad1);
IMPORT CONST Sleef_quad1 Sleef_subq1_u05purecfma(Sleef_quad1, Sleef_quad1);
IMPORT CONST Sleef_quad1 Sleef_mulq1_u05purecfma(Sleef_quad1, Sleef_quad1);
IMPORT CONST Sleef_quad1 Sleef_divq1_u05purecfma(Sleef_quad1, Sleef_quad1);
IMPORT CONST Sleef_quad1 Sleef_negq1_purecfma(Sleef_quad1);
IMPORT CONST Sleef_quad1 Sleef_sqrtq1_u05purecfma(Sleef_quad1);
IMPORT CONST int32_t Sleef_cmpltq1_purecfma(Sleef_quad1, Sleef_quad1);
IMPORT CONST int32_t Sleef_cmpleq1_purecfma(Sleef_quad1, Sleef_quad1);
IMPORT CONST int32_t Sleef_cmpgtq1_purecfma(Sleef_quad1, Sleef_quad1);
IMPORT CONST int32_t Sleef_cmpgeq1_purecfma(Sleef_quad1, Sleef_quad1);
IMPORT CONST int32_t Sleef_cmpeqq1_purecfma(Sleef_quad1, Sleef_quad1);
IMPORT CONST int32_t Sleef_cmpneqq1_purecfma(Sleef_quad1, Sleef_quad1);
IMPORT CONST int32_t Sleef_unordq1_purecfma(Sleef_quad1, Sleef_quad1);
IMPORT CONST double Sleef_cast_to_doubleq1_purecfma(Sleef_quad1);
IMPORT CONST Sleef_quad1 Sleef_cast_from_doubleq1_purecfma(double);
IMPORT CONST Sleef_quad1 Sleef_sinq1_u10purecfma(Sleef_quad1);
IMPORT CONST Sleef_quad1 Sleef_cosq1_u10purecfma(Sleef_quad1);
IMPORT CONST Sleef_quad1 Sleef_tanq1_u10purecfma(Sleef_quad1);
IMPORT CONST Sleef_quad1 Sleef_asinq1_u10purecfma(Sleef_quad1);
IMPORT CONST Sleef_quad1 Sleef_acosq1_u10purecfma(Sleef_quad1);
IMPORT CONST Sleef_quad1 Sleef_atanq1_u10purecfma(Sleef_quad1);
IMPORT CONST Sleef_quad1 Sleef_expq1_u10purecfma(Sleef_quad1);
IMPORT CONST Sleef_quad1 Sleef_exp2q1_u10purecfma(Sleef_quad1);
IMPORT CONST Sleef_quad1 Sleef_exp10q1_u10purecfma(Sleef_quad1);
IMPORT CONST Sleef_quad1 Sleef_expm1q1_u10purecfma(Sleef_quad1);
IMPORT CONST Sleef_quad1 Sleef_logq1_u10purecfma(Sleef_quad1);
IMPORT CONST Sleef_quad1 Sleef_log2q1_u10purecfma(Sleef_quad1);
IMPORT CONST Sleef_quad1 Sleef_log10q1_u10purecfma(Sleef_quad1);
IMPORT CONST Sleef_quad1 Sleef_log1pq1_u10purecfma(Sleef_quad1);
#endif
#ifdef __ARM_NEON
IMPORT CONST Sleef_quad2 Sleef_addq2_u05advsimd(Sleef_quad2, Sleef_quad2);
IMPORT CONST Sleef_quad2 Sleef_subq2_u05advsimd(Sleef_quad2, Sleef_quad2);
IMPORT CONST Sleef_quad2 Sleef_mulq2_u05advsimd(Sleef_quad2, Sleef_quad2);
IMPORT CONST Sleef_quad2 Sleef_divq2_u05advsimd(Sleef_quad2, Sleef_quad2);
IMPORT CONST Sleef_quad2 Sleef_negq2_advsimd(Sleef_quad2);
IMPORT CONST Sleef_quad2 Sleef_sqrtq2_u05advsimd(Sleef_quad2);
IMPORT CONST int32x2_t Sleef_cmpltq2_advsimd(Sleef_quad2, Sleef_quad2);
IMPORT CONST int32x2_t Sleef_cmpleq2_advsimd(Sleef_quad2, Sleef_quad2);
IMPORT CONST int32x2_t Sleef_cmpgtq2_advsimd(Sleef_quad2, Sleef_quad2);
IMPORT CONST int32x2_t Sleef_cmpgeq2_advsimd(Sleef_quad2, Sleef_quad2);
IMPORT CONST int32x2_t Sleef_cmpeqq2_advsimd(Sleef_quad2, Sleef_quad2);
IMPORT CONST int32x2_t Sleef_cmpneqq2_advsimd(Sleef_quad2, Sleef_quad2);
IMPORT CONST int32x2_t Sleef_unordq2_advsimd(Sleef_quad2, Sleef_quad2);
IMPORT CONST float64x2_t Sleef_cast_to_doubleq2_advsimd(Sleef_quad2);
IMPORT CONST Sleef_quad2 Sleef_cast_from_doubleq2_advsimd(float64x2_t);
IMPORT CONST Sleef_quad2 Sleef_sinq2_u10advsimd(Sleef_quad2);
IMPORT CONST Sleef_quad2 Sleef_cosq2_u10advsimd(Sleef_quad2);
IMPORT CONST Sleef_quad2 Sleef_tanq2_u10advsimd(Sleef_quad2);
IMPORT CONST Sleef_quad2 Sleef_asinq2_u10advsimd(Sleef_quad2);
IMPORT CONST Sleef_quad2 Sleef_acosq2_u10advsimd(Sleef_quad2);
IMPORT CONST Sleef_quad2 Sleef_atanq2_u10advsimd(Sleef_quad2);
IMPORT CONST Sleef_quad2 Sleef_expq2_u10advsimd(Sleef_quad2);
IMPORT CONST Sleef_quad2 Sleef_exp2q2_u10advsimd(Sleef_quad2);
IMPORT CONST Sleef_quad2 Sleef_exp10q2_u10advsimd(Sleef_quad2);
IMPORT CONST Sleef_quad2 Sleef_expm1q2_u10advsimd(Sleef_quad2);
IMPORT CONST Sleef_quad2 Sleef_logq2_u10advsimd(Sleef_quad2);
IMPORT CONST Sleef_quad2 Sleef_log2q2_u10advsimd(Sleef_quad2);
IMPORT CONST Sleef_quad2 Sleef_log10q2_u10advsimd(Sleef_quad2);
IMPORT CONST Sleef_quad2 Sleef_log1pq2_u10advsimd(Sleef_quad2);
#endif
#ifdef __cplusplus
}
#endif

#undef IMPORT
#endif // #ifndef __SLEEFQUAD_H__
